@@ -1108,12 +1108,13 @@ class Consumer(object):
         properties['headers']['X-Dropped-Timestamp'] = \
             datetime.datetime.utcnow().isoformat()
         properties['headers']['X-Original-Exchange'] = self._message.exchange
+        properties['headers']['X-Original-Queue'] = self._process.queue_name
 
         self._message.channel.basic_publish(
-            self._drop_exchange,
-            self._message.routing_key,
-            self._message.body,
-            pika.BasicProperties(**properties))
+            exchange=self._drop_exchange,
+            routing_key=self._message.routing_key,
+            body=self._message.body,
+            properties=pika.BasicProperties(**properties))
 
     def _republish_processing_error(self, error):
         """Republish the original message that was received because a
@@ -1132,6 +1133,9 @@ class Consumer(object):
         if 'headers' not in properties or not properties['headers']:
             properties['headers'] = {}
 
+        properties['headers']['X-Original-Exchange'] = self._message.exchange
+        properties['headers']['X-Original-Queue'] = self._process.queue_name
+
         if error:
             properties['headers']['X-Processing-Exception'] = error
 
@@ -1144,10 +1148,10 @@ class Consumer(object):
                 properties['headers'][_PROCESSING_EXCEPTIONS] = 1
 
         self._message.channel.basic_publish(
-            self._error_exchange,
-            self._message.routing_key,
-            self._message.body,
-            pika.BasicProperties(**properties))
+            exchange=self._error_exchange,
+            routing_key=self._message.routing_key,
+            body=self._message.body,
+            properties=pika.BasicProperties(**properties))
 
 
 class PublishingConsumer(Consumer):
